//
// 2002/3 Charlos Potma
//
// Program to control an Analog Devices AD9852 type DDS
//
// Schematic: see ad9852sch.gif
//
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Math, ExtCtrls;

type
  TForm_main = class(TForm)
    Edit_clock: TEdit;
    Label_clock: TLabel;
    Edit_frequency: TEdit;
    Label_frequency: TLabel;
    Edit_ftw: TEdit;
    Label_ftw: TLabel;
    Edit_ftwbits: TEdit;
    Label_ftwbits: TLabel;
    Label_ftw1: TLabel;
    Label_ftwbits1: TLabel;
    Button_masterreset: TButton;
    Button_send: TButton;
    Button_exit: TButton;
    Label_ftwhex: TLabel;
    Label_ftwhex1: TLabel;
    Edit_ftwhex: TEdit;
    Edit_portaddress: TEdit;
    Label_portaddress: TLabel;
    Label_status: TLabel;
    Edit_repeat: TEdit;
    Label_repeat: TLabel;
    Bits_sent: TMemo;
    Label_bitssent: TLabel;
    Label_ftwbytes: TLabel;
    Ftw_bytes: TMemo;
    Edit_level: TEdit;
    Label_level: TLabel;
    Label_header: TLabel;
    ComboBox_pllmultiplier: TComboBox;
    CheckBox_pllenabled: TCheckBox;
    Label_pllmultiplier: TLabel;
    Edit_clockeff: TEdit;
    Label_clockeff: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Bevel4: TBevel;
    Edit_controldac: TEdit;
    Label_controldac: TLabel;
    Label_twoscomplement: TLabel;
    CheckBox_fs: TCheckBox;
    CheckBox_max: TCheckBox;
    CheckBox_min: TCheckBox;
    CheckBox_controldac: TCheckBox;
    procedure Button_exitClick(Sender: TObject);
    procedure Button_sendClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button_masterresetClick(Sender: TObject);

    procedure sendCSB;
    procedure sendCSBNOT;
    procedure sendSCLK;
    procedure sendIOReset;
    procedure sendIOUD;
    procedure sendByte(senddata : Byte);
    procedure CheckBox_pllenabledClick(Sender: TObject);
    procedure ComboBox_pllmultiplierChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure CheckBox_fsClick(Sender: TObject);
    procedure CheckBox_maxClick(Sender: TObject);
    procedure CheckBox_minClick(Sender: TObject);
    procedure CheckBox_controldacClick(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

const               //From AD9852/4 datasheet rev.0 :
  MASTERRESET =  1; //bit 0 of data port, active on logic high
                    //pin 2 on db25 connector
  CSB         =  2; //bit 5 of data port, active on logic low
                    //pin 3 on db25 connector
  IORESET     =  4; //bit 3 of data port, active on logic high
                    //pin 4 on db25 connector
  SCLK        =  8; //bit 2 of data port, data is read on rising edge
                    //pin 5 on db25 connector
  SDO         = 16; //bit 1 of data port, data
                    //pin 6 on db25 connector
  IOUD        = 32; //bit 4 of data port, active on rising edge
                    //pin 7 on db25 connector
var
  Form_main   : TForm_main;
  portbits    : Byte;
  portaddress : Smallint;
  bytessent   : Integer;
  controldac  : Boolean;
implementation

{$R *.DFM}

{Source code for inpout32.dll.
Enables 32-bit Visual Basic programs to do direct port I/O
(Inp and Out) under Windows 95.
To be compiled with Borland's Delphi 2.0.
from: www.lvr.com }
procedure Out32(PortAddress:smallint;Value:smallint);
var
   ByteValue:Byte;
begin
     ByteValue:=Byte(Value);
     asm
        push dx
        mov dx,PortAddress
        mov al, ByteValue
        out dx,al
        pop dx
     end;
end;

function Inp32(PortAddress:smallint):smallint;
var
   ByteValue:byte;
begin
   asm
        push dx
        mov dx, PortAddress
        in al,dx
        mov ByteValue,al
        pop dx
    end;
    Inp32:=smallint(ByteValue) and $00FF;
end;

procedure TForm_main.Button_exitClick(Sender: TObject);
begin
  Close;
end;

procedure TForm_main.Button_sendClick(Sender: TObject);
const fourtyeight = 281474976710656; //2^48
var
  ftw                               : int64;
  buf                               : array[0..5] of byte;
  levelhigh, levellow               : byte;
  controldachigh, controldaclow     : byte;
  clock, clockeff, frequency        : double;
  bitnumber, bytenumber, sendrepeat : integer;
  hexstring                         : string;
  level                             : word;
  inputfault                        : boolean;
  pllbyte                           : byte;

begin
  Bytessent := 0;
  inputfault := False;

  //Check clock/pll
  if((strtofloat(Edit_clock.Text) > 300.0))then
  begin
    showmessage('DDS base clock must be >0 and <300 MHz');
    Edit_clock.SelectAll;
    Edit_clock.SetFocus;
    inputfault := True;
  end
  else
  begin
    clock := strtofloat(Edit_clock.Text);
    if Checkbox_pllenabled.Checked = True then
    begin
      if clock * (ComboBox_pllmultiplier.ItemIndex+4) > 300.0 then
      begin
        showmessage('(DDS base Clock) * (Pll multiplier) must be =< 300 MHz');
        ComboBox_pllmultiplier.SelectAll;
        ComboBox_pllmultiplier.SetFocus;
        inputfault := True;
      end
      else
      begin
        clockeff := clock * (ComboBox_pllmultiplier.ItemIndex+4);
        pllbyte := ComboBox_pllmultiplier.ItemIndex+4;
        if clockeff > 200 then
          pllbyte := pllbyte or 64;//set pll high bit
      end;
    end
    else
    begin
      pllbyte := 32;//meaning bypass pll
      clockeff := clock;
    end;
  end;

  //Check DDS output level
  if((strtoint(Edit_level.Text) < 0) or
   (strtoint(Edit_level.Text) > 4095))then
  begin
    showmessage('DDS output level is a 12-bit value between 0 and 4095');
    Edit_level.SelectAll;
    Edit_level.SetFocus;
    inputfault := True;
  end
  else
  begin
    level := strtoint(Edit_level.Text);
    levelhigh := (level and $0f00) shr 8; //bits 9..11
    levellow  := (level and $00ff);       //bits 0..8
  end;

  //Check control dac output level
  if((strtoint(Edit_controldac.Text) < 0) or
   (strtoint(Edit_controldac.Text) > 4095))then
  begin
    showmessage('Control DAC output level is a 12-bit value between 0 and 4095');
    Edit_controldac.SelectAll;
    Edit_controldac.SetFocus;
    inputfault := True;
  end
  else
  begin
    level := strtoint(Edit_controldac.Text);
    controldachigh := (level and $0f00) shr 8; //bits 9..11
    controldaclow  := (level and $00ff);       //bits 0..8
  end;
  //Check DDS frequency

  if((strtofloat(Edit_frequency.Text) < 0.0) or
   (strtofloat(Edit_frequency.Text) > (clockeff/2)))then
  begin
    showmessage('DDS Frequency must be > 0 and < DDS-Clock/2');
    Edit_frequency.SelectAll;
    Edit_frequency.SetFocus;
    inputfault := True;
  end
  else
    frequency := strtofloat(Edit_frequency.Text);
  if(inputfault = False) then
  begin
    //Calculate frequency tuning word
    ftw := trunc((frequency / clockeff) * fourtyeight);
    Edit_ftw.Text := inttostr(ftw);
    hexstring := inttohex(ftw, 12);
    //Calculate frequency tuning word in hexadecimal
    Edit_ftwhex.Text := '';
    for bitnumber := 1 to trunc(Length(hexstring)/2) do
    begin
      if bitnumber =  trunc(Length(hexstring)/2) then
      begin
        Edit_ftwhex.Text :=
        Copy(hexstring, Length(hexstring)-(bitnumber*2-1), 2) +
        Edit_ftwhex.Text;
      end
      else
      begin
        Edit_ftwhex.Text := '.' +
        Copy(hexstring, Length(hexstring)-(bitnumber*2-1), 2) +
        Edit_ftwhex.Text;
      end;
    end;
    //Calculate frequency tuning word in binary
    Edit_ftwbits.Text := '';
    for bytenumber := 0 to 5 do
      buf[bytenumber] := 0;
     //Test ftw bit-for-bit and fill ftwbits edit field accordingly
    for bitnumber := 1 to 48 do
    begin
      //Test ftw bit-for-bit and fill ftwbits edit field accordingly
      bytenumber := bitnumber div 8;
      if ((ftw and trunc(intpower(2,bitnumber-1)))
         = trunc(intpower(2,bitnumber-1))) then
      begin
        Edit_ftwbits.Text := '1' + Edit_ftwbits.Text;
        buf[bytenumber] := buf[bytenumber] +
          trunc(intpower(2,(bitnumber-1) - 8 * bytenumber));
      end
      else
        Edit_ftwbits.Text := '0' + Edit_ftwbits.Text;
        //add a decimal point separator every four bits for readability
      if ((bitnumber <> 48) and ((bitnumber mod 4) = 0))then
        Edit_ftwbits.Text := '.' + Edit_ftwbits.Text;
      Edit_ftwbits.Update;
    end;
    //Calculate the six bytes containing frequency tuning word
    for bytenumber := 0 to 5 do
    begin
      Ftw_bytes.lines.Add(inttostr(bytenumber)+' : '+
        inttostr(buf[bytenumber]));
    end;

    //Send data to DDS, repeat if necessary
    //to be able to monitor data stream with an oscilloscope
    for sendrepeat :=1 to strtoint(Edit_repeat.Text) do
    begin
      Label_status.Caption := 'send sequence: '+inttostr(sendrepeat);
      Label_status.Update;

      sendCSB;               //Chip select enabled
      sendIOReset;           //Send IO reset
//------------
      sendbyte(7);           //register #7 (dds setup)
      if(controldac)then
        sendbyte(16)         //control dac, dac and digital switched on,
                             //comparator powered down
                             //change '16' into '0' to switch comparator on
      else
        sendbyte(20);        //same, but control dac also powered down
      sendbyte(pllbyte);
      sendbyte(0);           //external io update
      sendbyte(96);          //bypass inv sync, osk en is active to enable
                             //output level to be set, osk int is off,
                             //msb first, sdo inactive
//------------
      sendbyte(2);           //register #2 (ftw 1)
      sendbyte(buf[5]);      //frequency word from calculated buf array
      sendbyte(buf[4]);
      sendbyte(buf[3]);
      sendbyte(buf[2]);
      sendbyte(buf[1]);
      sendbyte(buf[0]);
//------------
      sendbyte(8);              //register #8 (output level)
      sendbyte(levelhigh);      //from output level field
      sendbyte(levellow);       //
//------------
      sendbyte(11);             //register #B (control dac output level)
      sendbyte(controldachigh); //from control dac output level field
      sendbyte(controldaclow);  //note: in two's complement!

      sendIOUD;                 //IO Update pulse

      sendCSBNOT;               //Chip select disabled
      
      Label_status.Caption := 'Data sent to DDS';
    end;
  end
  else
  begin
    Label_status.Caption := 'Input parameter fault';
    Label_status.Update;
    inputfault := False;
  end;
end;

procedure TForm_main.FormCreate(Sender: TObject);
begin
  Form_main.Show;
  Sleep(100);
  //reset portbits to all zero's
  portbits := 0;
  portaddress := strtoint(Edit_portaddress.Text);
  Button_masterresetClick(Self);
  Edit_clockeff.Text := Edit_clock.Text;
  controldac := False;
end;

procedure TForm_main.Button_masterresetClick(Sender: TObject);
begin
  Beep;
  Label_status.Caption := 'Resetting DDS...';
  Label_status.Font.Color := ClRed;
  Label_status.Update;
  portbits := portbits or MASTERRESET;        //Masterreset high
  Out32(portaddress, portbits);
  Sleep(500); //milliseconds
  portbits := portbits and not(MASTERRESET);  //Masterreset low
  Out32(portaddress, portbits);
  Label_status.Caption := 'DDS is reset';
  Label_status.Font.Color := ClBlack;
  Label_status.Update;
end;

procedure TForm_main.sendCSB;
begin
  portbits := portbits and not(CSB);    //Chipselect low
  Out32(portaddress, portbits);
  //Sleep(1000);//!!!KANWEG
end;

procedure TForm_main.sendCSBNOT;
begin
  portbits := portbits or CSB;          //Chipselect high
  Out32(portaddress, portbits);
end;

procedure TForm_main.sendSCLK;
begin
  portbits := portbits or SCLK;         //Clock high
  Out32(portaddress, portbits);
  //Sleep(1);//!!!weg
  portbits := portbits and not(SCLK);   //Clock low
  Out32(portaddress, portbits);
end;

procedure TForm_main.sendIORESET;
begin
  portbits := portbits or IORESET;       //IOreset high
  Out32(portaddress, portbits);
  //Sleep(1000);//!!!weg
  portbits := portbits and not(IORESET); //IOReset low
  Out32(portaddress, portbits);
end;

procedure TForm_main.sendIOUD;
begin
  portbits := portbits or IOUD;          //IOUpdate high
  Out32(portaddress, portbits);
  portbits := portbits and not(IOUD);    //IOUpdate low
  Out32(portaddress, portbits);
end;

procedure TForm_main.sendByte(senddata : byte);
var
  bitnumber : integer;
  bitstring : string;
begin
  bitstring := inttostr(Bytessent) + ' : ';
  for bitnumber := 7 downto 0 do
  begin
    if ((senddata and trunc(intpower(2,bitnumber))) =
         trunc(intpower(2,bitnumber))) then
    begin
      portbits := portbits or SDO;       //Data line high
      Out32(portaddress, portbits);
      bitstring := bitstring + '1';
      //Sleep(1);
    end
    else
    begin
      portbits := portbits and not(SDO); //Data line low
      Out32(portaddress, portbits);
      bitstring := bitstring + '0';
      //Sleep(1);
    end;
    sendSCLK;                            //Toggle clock line
  end;
  Bits_sent.lines.Add(bitstring + '  dec: ' + inttostr(senddata));
  Bytessent := Bytessent + 1;
end;


procedure TForm_main.CheckBox_pllenabledClick(Sender: TObject);
begin
  if Checkbox_pllenabled.Checked then
  begin
    ComboBox_pllmultiplier.Enabled := True;
    ComboBox_pllmultiplier.Itemindex := 0;
    ComboBox_pllmultiplierChange(Self)
  end
  else
  begin
    ComboBox_pllmultiplier.Enabled := False;
    Edit_clockeff.Text := Edit_clock.Text;
  end;
end;

procedure TForm_main.ComboBox_pllmultiplierChange(Sender: TObject);
begin
  if Strtofloat(Edit_clock.Text) *
    (Combobox_pllmultiplier.ItemIndex+4) > 300.0 then
  begin
    showmessage('(DDS Base clock) x (Pll multiplier) must be =< 300 MHz');
    ComboBox_pllmultiplier.SelectAll;
    ComboBox_pllmultiplier.SetFocus;
  end
  else
  begin
    Edit_clockeff.Text := floattostr( Strtofloat(Edit_clock.Text) *
     (Combobox_pllmultiplier.ItemIndex+4));
  end;
end;

procedure TForm_main.Button1Click(Sender: TObject);
var i:integer;
begin
for i:=0 to 100000 do
  sendIOUD;
end;

procedure TForm_main.CheckBox_fsClick(Sender: TObject);
begin
//if FS is checked, set level to max and disable control,
//else enable control
  if CheckBox_fs.Checked = True then
  begin
    Edit_level.Text := inttostr(4095);
    Edit_level.Enabled := False;
  end
  else
    Edit_level.Enabled := True;
end;

procedure TForm_main.CheckBox_maxClick(Sender: TObject);
begin
//if 'max' is checked, set control dac level to max (in 2's complement
//and disable control and 'min'
//else enable control
  if CheckBox_max.Checked = True then
  begin
    Edit_controldac.Text := inttostr(2047);
    Edit_controldac.Enabled := False;
    CheckBox_min.Enabled := False;
    Checkbox_min.Checked := False;
  end
  else
  begin
    Edit_controldac.Enabled := True;
    CheckBox_max.Enabled := True;
    Checkbox_min.Enabled := True;
  end;
end;

procedure TForm_main.CheckBox_minClick(Sender: TObject);
begin
//if 'min' is checked, set control dac level to min (in 2's complement
//and disable control and 'max'
//else enable control
  if CheckBox_min.Checked = True then
  begin
    Edit_controldac.Text := inttostr(2048);
    Edit_controldac.Enabled := False;
    CheckBox_max.Enabled := False;
    Checkbox_max.Checked := False;
  end
  else
  begin
    Edit_controldac.Enabled := True;
    CheckBox_min.Enabled := True;
    Checkbox_max.Enabled := True;
  end;
end;

procedure TForm_main.CheckBox_controldacClick(Sender: TObject);
begin
if Checkbox_controldac.Checked = True then
begin
  Checkbox_max.Enabled    := True;
  Checkbox_min.Enabled    := True;
  Edit_controldac.Enabled := True;
  controldac := True;
end
else
begin
  Checkbox_max.Enabled    := False;
  Checkbox_min.Enabled    := False;
  Edit_controldac.Enabled := False;
  controldac := False;
end;
end;

end.
